@@ -20,6 +20,8 @@ SwitcherData *GetSwitcher()
 	return switcher;
 }
 
+AdvSceneSwitcher *AdvSceneSwitcher::window = nullptr;
+
 /******************************************************************************
  * Create the Advanced Scene Switcher settings window
  ******************************************************************************/
@@ -39,6 +41,7 @@ AdvSceneSwitcher::~AdvSceneSwitcher()
 {
 	if (switcher) {
 		switcher->settingsWindowOpened = false;
+		switcher->lastOpenedTab = ui->tabWidget->currentIndex();
 	}
 }
 
@@ -85,7 +88,9 @@ void AdvSceneSwitcher::loadUI()
 	setupTriggerTab();
 	setupMacroTab();
 
+	setDeprecationWarnings();
 	setTabOrder();
+	setCurrentTab();
 	restoreWindowGeo();
 	checkFirstTimeSetup();
 
@@ -340,7 +345,7 @@ bool SwitcherData::checkForMatch(OBSWeakSource &scene,
 	return match;
 }
 
-void switchScene(const sceneSwitchInfo &sceneSwitch)
+void switchScene(const sceneSwitchInfo &sceneSwitch, bool force)
 {
 	if (!sceneSwitch.scene && switcher->verbose) {
 		blog(LOG_INFO, "nothing to switch to");
@@ -350,7 +355,7 @@ void switchScene(const sceneSwitchInfo &sceneSwitch)
 	obs_source_t *source = obs_weak_source_get_source(sceneSwitch.scene);
 	obs_source_t *currentSource = obs_frontend_get_current_scene();
 
-	if (source && source != currentSource) {
+	if (source && (source != currentSource || force)) {
 		transitionData currentTransitionData;
 		setNextTransition(sceneSwitch, currentSource,
 				  currentTransitionData);
@@ -432,13 +437,6 @@ void SwitcherData::Stop()
 	server.stop();
 	client.disconnect();
 
-	for (auto &t : audioHelperThreads) {
-		if (t.joinable()) {
-			t.join();
-		}
-	}
-	audioHelperThreads.clear();
-
 	if (showSystemTrayNotifications) {
 		DisplayTrayMessage(
 			obs_module_text("AdvSceneSwitcher.pluginName"),
@@ -462,6 +460,19 @@ bool SwitcherData::sceneChangedDuringWait()
 	return (waitScene && currentSource != waitScene);
 }
 
+// Relies on the fact that switcher->currentScene will only be updated on event
+// OBS_FRONTEND_EVENT_SCENE_CHANGED but obs_frontend_get_current_scene() will
+// already return the scene to be transitioned to.
+bool SwitcherData::anySceneTransitionStarted()
+{
+	auto currentSceneSrouce = obs_frontend_get_current_scene();
+	auto currentScene = obs_source_get_weak_source(currentSceneSrouce);
+	bool ret = switcher->currentScene != currentScene;
+	obs_weak_source_release(currentScene);
+	obs_source_release(currentSceneSrouce);
+	return ret;
+}
+
 /******************************************************************************
  * OBS module setup
  ******************************************************************************/
@@ -574,6 +585,18 @@ void setStreamStopping()
 		std::chrono::high_resolution_clock::now();
 }
 
+void handleExit()
+{
+	switcher->obsIsShuttingDown = true;
+	if (switcher && switcher->obsIsShuttingDown &&
+	    switcher->shutdownConditionCount) {
+		switcher->Stop();
+		switcher->checkMacros();
+		switcher->runMacros();
+	}
+	FreeSceneSwitcher();
+}
+
 // Note to future self:
 // be careful using switcher->m here as there is potential for deadlocks when using
 // frontend functions such as obs_frontend_set_current_scene()
@@ -585,7 +608,7 @@ static void OBSEvent(enum obs_frontend_event event, void *switcher)
 
 	switch (event) {
 	case OBS_FRONTEND_EVENT_EXIT:
-		FreeSceneSwitcher();
+		handleExit();
 		break;
 	case OBS_FRONTEND_EVENT_SCENE_CHANGED:
 		handleSceneChange();
@@ -652,8 +675,6 @@ void LoadPlugins()
 	}
 }
 
-AdvSceneSwitcher *ssWindow;
-
 extern "C" void InitSceneSwitcher()
 {
 	blog(LOG_INFO, "version: %s", g_GIT_TAG);
@@ -671,15 +692,16 @@ extern "C" void InitSceneSwitcher()
 
 	auto cb = []() {
 		if (switcher->settingsWindowOpened) {
-			ssWindow->show();
-			ssWindow->raise();
-			ssWindow->activateWindow();
+			AdvSceneSwitcher::window->show();
+			AdvSceneSwitcher::window->raise();
+			AdvSceneSwitcher::window->activateWindow();
 		} else {
-			ssWindow =
+			AdvSceneSwitcher::window =
 				new AdvSceneSwitcher(static_cast<QMainWindow *>(
 					obs_frontend_get_main_window()));
-			ssWindow->setAttribute(Qt::WA_DeleteOnClose);
-			ssWindow->show();
+			AdvSceneSwitcher::window->setAttribute(
+				Qt::WA_DeleteOnClose);
+			AdvSceneSwitcher::window->show();
 		}
 	};
 
