@@ -72,33 +72,37 @@ bool MacroConditionVideo::CheckShouldBeSkipped()
 
 bool MacroConditionVideo::CheckCondition()
 {
-	bool match = false;
+	if (!_video.ValidSelection()) {
+		return false;
+	}
 
+	bool match = false;
 	if (CheckShouldBeSkipped()) {
 		return _lastMatchResult;
 	}
 
-	if (_screenshotData && _screenshotData->done) {
+	if (_screenshotData.done) {
 		match = Compare();
 		_lastMatchResult = match;
 
 		if (!requiresFileInput(_condition)) {
-			_matchImage = std::move(_screenshotData->image);
+			_matchImage = std::move(_screenshotData.image);
 		}
-		_screenshotData.reset(nullptr);
+		_getNextScreenshot = true;
 	} else {
 		match = _lastMatchResult;
 	}
 
-	GetScreenshot();
+	if (_getNextScreenshot) {
+		GetScreenshot();
+	}
 	return match;
 }
 
 bool MacroConditionVideo::Save(obs_data_t *obj)
 {
 	MacroCondition::Save(obj);
-	obs_data_set_string(obj, "videoSource",
-			    GetWeakSourceName(_videoSource).c_str());
+	_video.Save(obj);
 	obs_data_set_int(obj, "condition", static_cast<int>(_condition));
 	obs_data_set_string(obj, "filePath", _file.c_str());
 	obs_data_set_bool(obj, "usePatternForChangedCheck",
@@ -108,12 +112,12 @@ bool MacroConditionVideo::Save(obs_data_t *obj)
 	obs_data_set_string(obj, "modelDataPath", _modelDataPath.c_str());
 	obs_data_set_double(obj, "scaleFactor", _scaleFactor);
 	obs_data_set_int(obj, "minNeighbors", _minNeighbors);
-	obs_data_set_int(obj, "minSizeX", _minSizeX);
-	obs_data_set_int(obj, "minSizeY", _minSizeY);
-	obs_data_set_int(obj, "maxSizeX", _maxSizeX);
-	obs_data_set_int(obj, "maxSizeY", _maxSizeY);
+	_minSize.Save(obj, "minSize");
+	_maxSize.Save(obj, "maxSize");
 	obs_data_set_bool(obj, "throttleEnabled", _throttleEnabled);
 	obs_data_set_int(obj, "throttleCount", _throttleCount);
+	obs_data_set_bool(obj, "checkAreaEnabled", _checkAreaEnable);
+	_checkArea.Save(obj, "checkArea");
 	return true;
 }
 
@@ -131,8 +135,10 @@ bool isMinNeighborsValid(int minNeighbors)
 bool MacroConditionVideo::Load(obs_data_t *obj)
 {
 	MacroCondition::Load(obj);
-	const char *videoSourceName = obs_data_get_string(obj, "videoSource");
-	_videoSource = GetWeakSourceByName(videoSourceName);
+	_video.Load(obj);
+	if (obs_data_has_user_value(obj, "videoSource")) {
+		_video.Load(obj, "videoSource");
+	}
 	_condition =
 		static_cast<VideoCondition>(obs_data_get_int(obj, "condition"));
 	_file = obs_data_get_string(obj, "filePath");
@@ -149,13 +155,20 @@ bool MacroConditionVideo::Load(obs_data_t *obj)
 	if (!isMinNeighborsValid(_minNeighbors)) {
 		_minNeighbors = minMinNeighbors;
 	}
-	_minSizeX = obs_data_get_int(obj, "minSizeX");
-	_minSizeY = obs_data_get_int(obj, "minSizeY");
-	_maxSizeX = obs_data_get_int(obj, "maxSizeX");
-	_maxSizeY = obs_data_get_int(obj, "maxSizeY");
+	// TODO: Remove this fallback in future version
+	if (obs_data_has_user_value(obj, "minSizeX")) {
+		_minSize.width = obs_data_get_int(obj, "minSizeX");
+		_minSize.height = obs_data_get_int(obj, "minSizeY");
+		_maxSize.width = obs_data_get_int(obj, "maxSizeX");
+		_maxSize.height = obs_data_get_int(obj, "maxSizeY");
+	} else {
+		_minSize.Load(obj, "minSize");
+		_maxSize.Load(obj, "maxSize");
+	}
 	_throttleEnabled = obs_data_get_bool(obj, "throttleEnabled");
 	_throttleCount = obs_data_get_int(obj, "throttleCount");
-
+	_checkAreaEnable = obs_data_get_bool(obj, "checkAreaEnabled");
+	_checkArea.Load(obj, "checkArea");
 	if (requiresFileInput(_condition)) {
 		(void)LoadImageFromFile();
 	}
@@ -169,55 +182,16 @@ bool MacroConditionVideo::Load(obs_data_t *obj)
 
 std::string MacroConditionVideo::GetShortDesc()
 {
-	if (_videoSource) {
-		return GetWeakSourceName(_videoSource);
-	}
-	return "";
+	return _video.ToString();
 }
 
 void MacroConditionVideo::GetScreenshot()
 {
-	auto source = obs_weak_source_get_source(_videoSource);
-	_screenshotData = std::make_unique<AdvSSScreenshotObj>(source);
+	auto source = obs_weak_source_get_source(_video.GetVideo());
+	_screenshotData.~ScreenshotHelper();
+	new (&_screenshotData) ScreenshotHelper(source);
 	obs_source_release(source);
-}
-
-// Assumption is that QImage uses Format_RGBA8888.
-// Conversion from: https://github.com/dbzhang800/QtOpenCV
-cv::Mat QImageToMat(const QImage &img)
-{
-	if (img.isNull()) {
-		return cv::Mat();
-	}
-	return cv::Mat(img.height(), img.width(), CV_8UC(img.depth() / 8),
-		       (uchar *)img.bits(), img.bytesPerLine());
-}
-
-QImage MatToQImage(const cv::Mat &mat)
-{
-	if (mat.empty()) {
-		return QImage();
-	}
-	return QImage(mat.data, mat.cols, mat.rows, mat.step,
-		      QImage::Format::Format_RGBA8888);
-}
-
-PatternMatchData createPatternData(QImage &pattern)
-{
-	PatternMatchData data;
-	if (pattern.isNull()) {
-		return data;
-	}
-
-	data.rgbaPattern = QImageToMat(pattern);
-	std::vector<cv::Mat1b> rgbaChannelsPattern;
-	cv::split(data.rgbaPattern, rgbaChannelsPattern);
-	std::vector<cv::Mat1b> rgbChanlesPattern(
-		rgbaChannelsPattern.begin(), rgbaChannelsPattern.begin() + 3);
-	cv::merge(rgbChanlesPattern, data.rgbPattern);
-	cv::threshold(rgbaChannelsPattern[3], data.mask, 0, 255,
-		      cv::THRESH_BINARY);
-	return data;
+	_getNextScreenshot = false;
 }
 
 bool MacroConditionVideo::LoadImageFromFile()
@@ -241,51 +215,10 @@ bool MacroConditionVideo::LoadModelData(std::string &path)
 	return !_objectCascade.empty();
 }
 
-void matchPattern(QImage &img, PatternMatchData &patternData, double threshold,
-		  cv::Mat &result, bool useAlphaAsMask = true)
-{
-	if (img.isNull() || patternData.rgbaPattern.empty()) {
-		return;
-	}
-	if (img.height() < patternData.rgbaPattern.rows ||
-	    img.width() < patternData.rgbaPattern.cols) {
-		return;
-	}
-
-	auto i = QImageToMat(img);
-
-	if (useAlphaAsMask) {
-		std::vector<cv::Mat1b> rgbaChannelsImage;
-		cv::split(i, rgbaChannelsImage);
-		std::vector<cv::Mat1b> rgbChanlesImage(
-			rgbaChannelsImage.begin(),
-			rgbaChannelsImage.begin() + 3);
-
-		cv::Mat3b rgbImage;
-		cv::merge(rgbChanlesImage, rgbImage);
-
-		cv::matchTemplate(rgbImage, patternData.rgbPattern, result,
-				  cv::TM_CCORR_NORMED, patternData.mask);
-		cv::threshold(result, result, threshold, 0, cv::THRESH_TOZERO);
-
-	} else {
-		cv::matchTemplate(i, patternData.rgbaPattern, result,
-				  cv::TM_CCOEFF_NORMED);
-		cv::threshold(result, result, threshold, 0, cv::THRESH_TOZERO);
-	}
-}
-
-void matchPattern(QImage &img, QImage &pattern, double threshold,
-		  cv::Mat &result, bool useAlphaAsMask)
-{
-	auto data = createPatternData(pattern);
-	matchPattern(img, data, threshold, result, useAlphaAsMask);
-}
-
 bool MacroConditionVideo::ScreenshotContainsPattern()
 {
 	cv::Mat result;
-	matchPattern(_screenshotData->image, _patternData, _patternThreshold,
+	matchPattern(_screenshotData.image, _patternData, _patternThreshold,
 		     result, _useAlphaAsMask);
 	return countNonZero(result) > 0;
 }
@@ -295,53 +228,40 @@ bool MacroConditionVideo::OutputChanged()
 	if (_usePatternForChangedCheck) {
 		cv::Mat result;
 		_patternData = createPatternData(_matchImage);
-		matchPattern(_screenshotData->image, _patternData,
+		matchPattern(_screenshotData.image, _patternData,
 			     _patternThreshold, result, _useAlphaAsMask);
 		return countNonZero(result) == 0;
 	}
-	return _screenshotData->image != _matchImage;
-}
-
-std::vector<cv::Rect> matchObject(QImage &img, cv::CascadeClassifier &cascade,
-				  double scaleFactor, int minNeighbors,
-				  cv::Size minSize, cv::Size maxSize)
-{
-	if (img.isNull() || cascade.empty()) {
-		return {};
-	}
-
-	auto i = QImageToMat(img);
-	cv::Mat frameGray;
-	cv::cvtColor(i, frameGray, cv::COLOR_BGR2GRAY);
-	equalizeHist(frameGray, frameGray);
-	std::vector<cv::Rect> objects;
-	cascade.detectMultiScale(frameGray, objects, scaleFactor, minNeighbors,
-				 0, minSize, maxSize);
-	return objects;
+	return _screenshotData.image != _matchImage;
 }
 
 bool MacroConditionVideo::ScreenshotContainsObject()
 {
-	auto objects = matchObject(_screenshotData->image, _objectCascade,
-				   _scaleFactor, _minNeighbors,
-				   {_minSizeX, _minSizeY},
-				   {_maxSizeX, _maxSizeY});
+	auto objects = matchObject(_screenshotData.image, _objectCascade,
+				   _scaleFactor, _minNeighbors, _minSize.CV(),
+				   _maxSize.CV());
 	return objects.size() > 0;
 }
 
 bool MacroConditionVideo::Compare()
 {
+	if (_checkAreaEnable && _condition != VideoCondition::NO_IMAGE) {
+		_screenshotData.image = _screenshotData.image.copy(
+			_checkArea.x, _checkArea.y, _checkArea.width,
+			_checkArea.height);
+	}
+
 	switch (_condition) {
 	case VideoCondition::MATCH:
-		return _screenshotData->image == _matchImage;
+		return _screenshotData.image == _matchImage;
 	case VideoCondition::DIFFER:
-		return _screenshotData->image != _matchImage;
+		return _screenshotData.image != _matchImage;
 	case VideoCondition::HAS_CHANGED:
 		return OutputChanged();
 	case VideoCondition::HAS_NOT_CHANGED:
 		return !OutputChanged();
 	case VideoCondition::NO_IMAGE:
-		return _screenshotData->image.isNull();
+		return _screenshotData.image.isNull();
 	case VideoCondition::PATTERN:
 		return ScreenshotContainsPattern();
 	case VideoCondition::OBJECT:
@@ -359,108 +279,62 @@ static inline void populateConditionSelection(QComboBox *list)
 	}
 }
 
-ThresholdSlider::ThresholdSlider(double min, double max, const QString &label,
-				 const QString &description, QWidget *parent)
-	: QWidget(parent)
-{
-	_slider = new QSlider();
-	_slider->setOrientation(Qt::Horizontal);
-	_slider->setRange(min * _scale, max * _scale);
-	_value = new QLabel();
-	QString labelText = label + QString("0.");
-	for (int i = 0; i < _precision; i++) {
-		labelText.append(QString("0"));
-	}
-	_value->setText(labelText);
-	connect(_slider, SIGNAL(valueChanged(int)), this,
-		SLOT(NotifyValueChanged(int)));
-	QVBoxLayout *mainLayout = new QVBoxLayout();
-	QHBoxLayout *sliderLayout = new QHBoxLayout();
-	sliderLayout->addWidget(_value);
-	sliderLayout->addWidget(_slider);
-	mainLayout->addLayout(sliderLayout);
-	if (!description.isEmpty()) {
-		mainLayout->addWidget(new QLabel(description));
-	}
-	mainLayout->setContentsMargins(0, 0, 0, 0);
-	setLayout(mainLayout);
-}
-
-void ThresholdSlider::SetDoubleValue(double value)
-{
-	_slider->setValue(value * _scale);
-	SetDoubleValueText(value);
-}
-
-void ThresholdSlider::NotifyValueChanged(int value)
-{
-	double doubleValue = value / _scale;
-	SetDoubleValueText(doubleValue);
-	emit DoubleValueChanged(doubleValue);
-}
-
-void ThresholdSlider::SetDoubleValueText(double value)
-{
-	QString labelText = _value->text();
-	labelText.chop(_precision + 2); // 2 for the part left of the "."
-	labelText.append(QString::number(value, 'f', _precision));
-	_value->setText(labelText);
-}
-
 MacroConditionVideoEdit::MacroConditionVideoEdit(
 	QWidget *parent, std::shared_ptr<MacroConditionVideo> entryData)
-	: QWidget(parent)
+	: QWidget(parent),
+	  _videoSelection(new VideoSelectionWidget(this)),
+	  _condition(new QComboBox()),
+	  _usePatternForChangedCheck(new QCheckBox(obs_module_text(
+		  "AdvSceneSwitcher.condition.video.usePatternForChangedCheck"))),
+	  _imagePath(new FileSelection()),
+	  _patternThreshold(new ThresholdSlider(
+		  0., 1.,
+		  obs_module_text(
+			  "AdvSceneSwitcher.condition.video.patternThreshold"),
+		  obs_module_text(
+			  "AdvSceneSwitcher.condition.video.patternThresholdDescription"))),
+	  _useAlphaAsMask(new QCheckBox(obs_module_text(
+		  "AdvSceneSwitcher.condition.video.patternThresholdUseAlphaAsMask"))),
+	  _modelDataPath(new FileSelection()),
+	  _modelPathLayout(new QHBoxLayout),
+	  _objectScaleThreshold(new ThresholdSlider(
+		  1.1, 5.,
+		  obs_module_text(
+			  "AdvSceneSwitcher.condition.video.objectScaleThreshold"),
+		  obs_module_text(
+			  "AdvSceneSwitcher.condition.video.objectScaleThresholdDescription"))),
+	  _neighborsControlLayout(new QHBoxLayout),
+	  _minNeighbors(new QSpinBox()),
+	  _minNeighborsDescription(new QLabel(obs_module_text(
+		  "AdvSceneSwitcher.condition.video.minNeighborDescription"))),
+	  _sizeLayout(new QHBoxLayout()),
+	  _minSize(new SizeSelection(0, 1024)),
+	  _maxSize(new SizeSelection(0, 4096)),
+	  _checkAreaControlLayout(new QHBoxLayout),
+	  _checkAreaEnable(new QCheckBox()),
+	  _checkArea(new AreaSelection(0, 99999)),
+	  _selectArea(new QPushButton(obs_module_text(
+		  "AdvSceneSwitcher.condition.video.selectArea"))),
+	  _throttleControlLayout(new QHBoxLayout),
+	  _throttleEnable(new QCheckBox()),
+	  _throttleCount(new QSpinBox()),
+	  _showMatch(new QPushButton(obs_module_text(
+		  "AdvSceneSwitcher.condition.video.showMatch"))),
+	  _previewDialog(this, entryData.get(), &GetSwitcher()->m)
 {
-	_videoSelection = new QComboBox();
-	_condition = new QComboBox();
-
-	_imagePath = new FileSelection();
 	_imagePath->Button()->disconnect();
-	_usePatternForChangedCheck = new QCheckBox(obs_module_text(
-		"AdvSceneSwitcher.condition.video.usePatternForChangedCheck"));
 	_usePatternForChangedCheck->setToolTip(obs_module_text(
 		"AdvSceneSwitcher.condition.video.usePatternForChangedCheck.tooltip"));
-	_patternThreshold = new ThresholdSlider(
-		0., 1.,
-		obs_module_text(
-			"AdvSceneSwitcher.condition.video.patternThreshold"),
-		obs_module_text(
-			"AdvSceneSwitcher.condition.video.patternThresholdDescription"));
-	_useAlphaAsMask = new QCheckBox(obs_module_text(
-		"AdvSceneSwitcher.condition.video.patternThresholdUseAlphaAsMask"));
-
-	_modelDataPath = new FileSelection();
-	_objectScaleThreshold = new ThresholdSlider(
-		1.1, 5.,
-		obs_module_text(
-			"AdvSceneSwitcher.condition.video.objectScaleThreshold"),
-		obs_module_text(
-			"AdvSceneSwitcher.condition.video.objectScaleThresholdDescription"));
-	_minNeighbors = new QSpinBox();
 	_minNeighbors->setMinimum(minMinNeighbors);
 	_minNeighbors->setMaximum(maxMinNeighbors);
-	_minNeighborsDescription = new QLabel(obs_module_text(
-		"AdvSceneSwitcher.condition.video.minNeighborDescription"));
-	_minSizeX = new QSpinBox();
-	_minSizeY = new QSpinBox();
-	_minSizeX->setMaximum(1024);
-	_minSizeY->setMaximum(1024);
-	_maxSizeX = new QSpinBox();
-	_maxSizeY = new QSpinBox();
-	_maxSizeX->setMaximum(4096);
-	_maxSizeY->setMaximum(4096);
-
-	_throttleEnable = new QCheckBox();
-	_throttleCount = new QSpinBox();
 	_throttleCount->setMinimum(1 * GetSwitcher()->interval);
 	_throttleCount->setMaximum(10 * GetSwitcher()->interval);
 	_throttleCount->setSingleStep(GetSwitcher()->interval);
-	_showMatch = new QPushButton(
-		obs_module_text("AdvSceneSwitcher.condition.video.showMatch"));
 
 	QWidget::connect(_videoSelection,
-			 SIGNAL(currentTextChanged(const QString &)), this,
-			 SLOT(SourceChanged(const QString &)));
+			 SIGNAL(VideoSelectionChange(const VideoSelection &)),
+			 this,
+			 SLOT(VideoSelectionChanged(const VideoSelection &)));
 	QWidget::connect(_condition, SIGNAL(currentIndexChanged(int)), this,
 			 SLOT(ConditionChanged(int)));
 	QWidget::connect(_imagePath, SIGNAL(PathChanged(const QString &)), this,
@@ -478,14 +352,14 @@ MacroConditionVideoEdit::MacroConditionVideoEdit(
 			 SLOT(ObjectScaleThresholdChanged(double)));
 	QWidget::connect(_minNeighbors, SIGNAL(valueChanged(int)), this,
 			 SLOT(MinNeighborsChanged(int)));
-	QWidget::connect(_minSizeX, SIGNAL(valueChanged(int)), this,
-			 SLOT(MinSizeXChanged(int)));
-	QWidget::connect(_minSizeY, SIGNAL(valueChanged(int)), this,
-			 SLOT(MinSizeYChanged(int)));
-	QWidget::connect(_maxSizeX, SIGNAL(valueChanged(int)), this,
-			 SLOT(MaxSizeXChanged(int)));
-	QWidget::connect(_maxSizeY, SIGNAL(valueChanged(int)), this,
-			 SLOT(MaxSizeYChanged(int)));
+	QWidget::connect(_minSize, SIGNAL(SizeChanged(advss::Size)), this,
+			 SLOT(MinSizeChanged(advss::Size)));
+	QWidget::connect(_maxSize, SIGNAL(SizeChanged(advss::Size)), this,
+			 SLOT(MaxSizeChanged(advss::Size)));
+	QWidget::connect(_checkAreaEnable, SIGNAL(stateChanged(int)), this,
+			 SLOT(CheckAreaEnableChanged(int)));
+	QWidget::connect(_checkArea, SIGNAL(AreaChanged(advss::Area)), this,
+			 SLOT(CheckAreaChanged(advss::Area)));
 	QWidget::connect(_modelDataPath, SIGNAL(PathChanged(const QString &)),
 			 this, SLOT(ModelPathChanged(const QString &)));
 	QWidget::connect(_throttleEnable, SIGNAL(stateChanged(int)), this,
@@ -494,8 +368,11 @@ MacroConditionVideoEdit::MacroConditionVideoEdit(
 			 SLOT(ThrottleCountChanged(int)));
 	QWidget::connect(_showMatch, SIGNAL(clicked()), this,
 			 SLOT(ShowMatchClicked()));
+	QWidget::connect(&_previewDialog, SIGNAL(SelectionAreaChanged(QRect)),
+			 this, SLOT(CheckAreaChanged(QRect)));
+	QWidget::connect(_selectArea, SIGNAL(clicked()), this,
+			 SLOT(SelectAreaClicked()));
 
-	populateVideoSelection(_videoSelection);
 	populateConditionSelection(_condition);
 
 	QHBoxLayout *entryLine1Layout = new QHBoxLayout;
@@ -504,43 +381,47 @@ MacroConditionVideoEdit::MacroConditionVideoEdit(
 		{"{{condition}}", _condition},
 		{"{{imagePath}}", _imagePath},
 		{"{{minNeighbors}}", _minNeighbors},
-		{"{{minSizeX}}", _minSizeX},
-		{"{{minSizeY}}", _minSizeY},
-		{"{{maxSizeX}}", _maxSizeX},
-		{"{{maxSizeY}}", _maxSizeY},
+		{"{{minSize}}", _minSize},
+		{"{{maxSize}}", _maxSize},
 		{"{{modelDataPath}}", _modelDataPath},
 		{"{{throttleEnable}}", _throttleEnable},
 		{"{{throttleCount}}", _throttleCount},
+		{"{{checkAreaEnable}}", _checkAreaEnable},
+		{"{{checkArea}}", _checkArea},
+		{"{{selectArea}}", _selectArea},
 	};
 	placeWidgets(obs_module_text("AdvSceneSwitcher.condition.video.entry"),
 		     entryLine1Layout, widgetPlaceholders);
-
-	_modelPathLayout = new QHBoxLayout;
 	placeWidgets(
 		obs_module_text(
 			"AdvSceneSwitcher.condition.video.entry.modelPath"),
 		_modelPathLayout, widgetPlaceholders);
-
-	_neighborsControlLayout = new QHBoxLayout;
 	placeWidgets(
 		obs_module_text(
 			"AdvSceneSwitcher.condition.video.entry.minNeighbor"),
 		_neighborsControlLayout, widgetPlaceholders);
-
-	_minSizeControlLayout = new QHBoxLayout;
-	placeWidgets(obs_module_text(
-			     "AdvSceneSwitcher.condition.video.entry.minSize"),
-		     _minSizeControlLayout, widgetPlaceholders);
-
-	_maxSizeControlLayout = new QHBoxLayout;
-	placeWidgets(obs_module_text(
-			     "AdvSceneSwitcher.condition.video.entry.maxSize"),
-		     _maxSizeControlLayout, widgetPlaceholders);
-
-	_throttleControlLayout = new QHBoxLayout;
 	placeWidgets(obs_module_text(
 			     "AdvSceneSwitcher.condition.video.entry.throttle"),
 		     _throttleControlLayout, widgetPlaceholders);
+	placeWidgets(
+		obs_module_text(
+			"AdvSceneSwitcher.condition.video.entry.checkArea"),
+		_checkAreaControlLayout, widgetPlaceholders);
+
+	QGridLayout *sizeGrid = new QGridLayout;
+	sizeGrid->addWidget(
+		new QLabel(obs_module_text(
+			"AdvSceneSwitcher.condition.video.minSize")),
+		0, 0);
+	sizeGrid->addWidget(_minSize, 0, 1);
+	sizeGrid->addWidget(
+		new QLabel(obs_module_text(
+			"AdvSceneSwitcher.condition.video.maxSize")),
+		1, 0);
+	sizeGrid->addWidget(_maxSize, 1, 1);
+	_sizeLayout->addLayout(sizeGrid);
+	_sizeLayout->addStretch();
+	_sizeLayout->setContentsMargins(0, 0, 0, 0);
 
 	QHBoxLayout *showMatchLayout = new QHBoxLayout;
 	showMatchLayout->addWidget(_showMatch);
@@ -554,10 +435,10 @@ MacroConditionVideoEdit::MacroConditionVideoEdit(
 	mainLayout->addWidget(_objectScaleThreshold);
 	mainLayout->addLayout(_neighborsControlLayout);
 	mainLayout->addWidget(_minNeighborsDescription);
-	mainLayout->addLayout(_minSizeControlLayout);
-	mainLayout->addLayout(_maxSizeControlLayout);
+	mainLayout->addLayout(_sizeLayout);
 	mainLayout->addLayout(showMatchLayout);
 	mainLayout->addLayout(_throttleControlLayout);
+	mainLayout->addLayout(_checkAreaControlLayout);
 	setLayout(mainLayout);
 
 	_entryData = entryData;
@@ -591,14 +472,14 @@ void MacroConditionVideoEdit::UpdatePreviewTooltip()
 	this->setToolTip(html);
 }
 
-void MacroConditionVideoEdit::SourceChanged(const QString &text)
+void MacroConditionVideoEdit::VideoSelectionChanged(const VideoSelection &v)
 {
 	if (_loading || !_entryData) {
 		return;
 	}
 
 	std::lock_guard<std::mutex> lock(GetSwitcher()->m);
-	_entryData->_videoSource = GetWeakSourceByQString(text);
+	_entryData->_video = v;
 	_entryData->ResetLastMatch();
 	emit HeaderInfoChanged(
 		QString::fromStdString(_entryData->GetShortDesc()));
@@ -653,7 +534,7 @@ void MacroConditionVideoEdit::ImageBrowseButtonClicked()
 	QString path;
 	bool useExistingFile = false;
 	// Ask whether to create screenshot or to select existing file
-	if (_entryData->_videoSource) {
+	if (_entryData->_video.ValidSelection()) {
 		QMessageBox msgBox(
 			QMessageBox::Question,
 			obs_module_text("AdvSceneSwitcher.windowTitle"),
@@ -680,9 +561,9 @@ void MacroConditionVideoEdit::ImageBrowseButtonClicked()
 		}
 
 	} else {
-		auto source =
-			obs_weak_source_get_source(_entryData->_videoSource);
-		auto screenshot = std::make_unique<AdvSSScreenshotObj>(source);
+		auto source = obs_weak_source_get_source(
+			_entryData->_video.GetVideo());
+		ScreenshotHelper screenshot(source);
 		obs_source_release(source);
 
 		path = QFileDialog::getSaveFileName(this);
@@ -693,16 +574,22 @@ void MacroConditionVideoEdit::ImageBrowseButtonClicked()
 		if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
 			return;
 		}
-		if (!screenshot->done) { // Screenshot usually completed by now
+		if (!screenshot.done) { // Screenshot usually completed by now
 			std::this_thread::sleep_for(std::chrono::seconds(1));
 		}
-		if (!screenshot->done) {
+		if (!screenshot.done) {
 			DisplayMessage(obs_module_text(
 				"AdvSceneSwitcher.condition.video.screenshotFail"));
 			return;
 		}
-
-		screenshot->image.save(path);
+		if (_entryData->_checkAreaEnable) {
+			screenshot.image = screenshot.image.copy(
+				_entryData->_checkArea.x,
+				_entryData->_checkArea.y,
+				_entryData->_checkArea.width,
+				_entryData->_checkArea.height);
+		}
+		screenshot.image.save(path);
 	}
 	_imagePath->SetPath(path);
 	ImagePathChanged(path);
@@ -761,44 +648,53 @@ void MacroConditionVideoEdit::MinNeighborsChanged(int value)
 	_entryData->_minNeighbors = value;
 }
 
-void MacroConditionVideoEdit::MinSizeXChanged(int value)
+void MacroConditionVideoEdit::MinSizeChanged(advss::Size value)
 {
 	if (_loading || !_entryData) {
 		return;
 	}
 
 	std::lock_guard<std::mutex> lock(GetSwitcher()->m);
-	_entryData->_minSizeX = value;
+	_entryData->_minSize = value;
 }
 
-void MacroConditionVideoEdit::MinSizeYChanged(int value)
+void MacroConditionVideoEdit::MaxSizeChanged(advss::Size value)
 {
 	if (_loading || !_entryData) {
 		return;
 	}
 
 	std::lock_guard<std::mutex> lock(GetSwitcher()->m);
-	_entryData->_minSizeY = value;
+	_entryData->_maxSize = value;
 }
 
-void MacroConditionVideoEdit::MaxSizeXChanged(int value)
+void MacroConditionVideoEdit::CheckAreaEnableChanged(int value)
 {
 	if (_loading || !_entryData) {
 		return;
 	}
 
 	std::lock_guard<std::mutex> lock(GetSwitcher()->m);
-	_entryData->_maxSizeX = value;
+	_entryData->_checkAreaEnable = value;
+	_checkArea->setEnabled(value);
+	_selectArea->setEnabled(value);
 }
 
-void MacroConditionVideoEdit::MaxSizeYChanged(int value)
+void MacroConditionVideoEdit::CheckAreaChanged(advss::Area value)
 {
 	if (_loading || !_entryData) {
 		return;
 	}
 
 	std::lock_guard<std::mutex> lock(GetSwitcher()->m);
-	_entryData->_maxSizeY = value;
+	_entryData->_checkArea = value;
+}
+
+void MacroConditionVideoEdit::CheckAreaChanged(QRect rect)
+{
+	advss::Area area{rect.topLeft().x(), rect.y(), rect.width(),
+			 rect.height()};
+	_checkArea->SetArea(area);
 }
 
 void MacroConditionVideoEdit::ThrottleEnableChanged(int value)
@@ -822,83 +718,20 @@ void MacroConditionVideoEdit::ThrottleCountChanged(int value)
 	_entryData->_throttleCount = value / GetSwitcher()->interval;
 }
 
-QImage markPatterns(cv::Mat &matchResult, QImage &image, QImage &pattern)
-{
-	auto matchImg = QImageToMat(image);
-	for (int row = 0; row < matchResult.rows - 1; row++) {
-		for (int col = 0; col < matchResult.cols - 1; col++) {
-			if (matchResult.at<float>(row, col) != 0.0) {
-				rectangle(matchImg, {col, row},
-					  cv::Point(col + pattern.width(),
-						    row + pattern.height()),
-					  cv::Scalar(255, 0, 0, 255), 2, 8, 0);
-			}
-		}
-	}
-	return MatToQImage(matchImg);
-}
-
-QImage markObjects(QImage &image, std::vector<cv::Rect> &objects)
-{
-	auto frame = QImageToMat(image);
-	for (size_t i = 0; i < objects.size(); i++) {
-		rectangle(frame, cv::Point(objects[i].x, objects[i].y),
-			  cv::Point(objects[i].x + objects[i].width,
-				    objects[i].y + objects[i].height),
-			  cv::Scalar(255, 0, 0, 255), 2, 8, 0);
-	}
-	return MatToQImage(frame);
-}
-
 void MacroConditionVideoEdit::ShowMatchClicked()
 {
-	auto source = obs_weak_source_get_source(_entryData->_videoSource);
-	auto screenshot = std::make_unique<AdvSSScreenshotObj>(source);
-	obs_source_release(source);
-	if (!screenshot->done) {
-		std::this_thread::sleep_for(std::chrono::seconds(1));
-	}
-	if (!screenshot->done) {
-		DisplayMessage(obs_module_text(
-			"AdvSceneSwitcher.condition.video.screenshotFail"));
-		return;
-	}
+	_previewDialog.show();
+	_previewDialog.raise();
+	_previewDialog.activateWindow();
+	_previewDialog.ShowMatch();
+}
 
-	QImage markedIamge;
-	if (_entryData->_condition == VideoCondition::PATTERN) {
-		cv::Mat result;
-		QImage pattern = _entryData->GetMatchImage();
-		matchPattern(screenshot->image, pattern,
-			     _entryData->_patternThreshold, result,
-			     _entryData->_useAlphaAsMask);
-		if (countNonZero(result) == 0) {
-			DisplayMessage(obs_module_text(
-				"AdvSceneSwitcher.condition.video.patternMatchFail"));
-			return;
-		}
-		markedIamge = markPatterns(result, screenshot->image, pattern);
-	} else if (_entryData->_condition == VideoCondition::OBJECT) {
-		auto objects = matchObject(
-			screenshot->image, _entryData->_objectCascade,
-			_entryData->_scaleFactor, _entryData->_minNeighbors,
-			{_entryData->_minSizeX, _entryData->_minSizeY},
-			{_entryData->_maxSizeX, _entryData->_maxSizeY});
-		if (objects.empty()) {
-			DisplayMessage(obs_module_text(
-				"AdvSceneSwitcher.condition.video.objectMatchFail"));
-			return;
-		}
-		markedIamge = markObjects(screenshot->image, objects);
-	}
-
-	QLabel *label = new QLabel;
-	label->setPixmap(QPixmap::fromImage(markedIamge));
-	QVBoxLayout *layout = new QVBoxLayout;
-	layout->addWidget(label);
-	QDialog dialog;
-	dialog.setLayout(layout);
-	dialog.setWindowTitle("Advanced Scene Switcher");
-	dialog.exec();
+void MacroConditionVideoEdit::SelectAreaClicked()
+{
+	_previewDialog.show();
+	_previewDialog.raise();
+	_previewDialog.activateWindow();
+	_previewDialog.SelectArea();
 }
 
 void MacroConditionVideoEdit::ModelPathChanged(const QString &text)
@@ -949,6 +782,11 @@ bool patternControlIsOptional(VideoCondition cond)
 	       cond == VideoCondition::HAS_NOT_CHANGED;
 }
 
+bool needsAreaControls(VideoCondition cond)
+{
+	return cond != VideoCondition::NO_IMAGE;
+}
+
 void MacroConditionVideoEdit::SetWidgetVisibility()
 {
 	_imagePath->setVisible(requiresFileInput(_entryData->_condition));
@@ -964,14 +802,14 @@ void MacroConditionVideoEdit::SetWidgetVisibility()
 			 needsObjectControls(_entryData->_condition));
 	_minNeighborsDescription->setVisible(
 		needsObjectControls(_entryData->_condition));
-	setLayoutVisible(_minSizeControlLayout,
-			 needsObjectControls(_entryData->_condition));
-	setLayoutVisible(_maxSizeControlLayout,
+	setLayoutVisible(_sizeLayout,
 			 needsObjectControls(_entryData->_condition));
 	setLayoutVisible(_modelPathLayout,
 			 needsObjectControls(_entryData->_condition));
 	setLayoutVisible(_throttleControlLayout,
 			 needsThrottleControls(_entryData->_condition));
+	setLayoutVisible(_checkAreaControlLayout,
+			 needsAreaControls(_entryData->_condition));
 
 	if (_entryData->_condition == VideoCondition::HAS_CHANGED ||
 	    _entryData->_condition == VideoCondition::HAS_NOT_CHANGED) {
@@ -988,8 +826,7 @@ void MacroConditionVideoEdit::UpdateEntryData()
 		return;
 	}
 
-	_videoSelection->setCurrentText(
-		GetWeakSourceName(_entryData->_videoSource).c_str());
+	_videoSelection->SetVideoSelection(_entryData->_video);
 	_condition->setCurrentIndex(static_cast<int>(_entryData->_condition));
 	_imagePath->SetPath(QString::fromStdString(_entryData->_file));
 	_usePatternForChangedCheck->setChecked(
@@ -999,12 +836,13 @@ void MacroConditionVideoEdit::UpdateEntryData()
 	_modelDataPath->SetPath(_entryData->GetModelDataPath().c_str());
 	_objectScaleThreshold->SetDoubleValue(_entryData->_scaleFactor);
 	_minNeighbors->setValue(_entryData->_minNeighbors);
-	_minSizeX->setValue(_entryData->_minSizeX);
-	_minSizeY->setValue(_entryData->_minSizeY);
-	_maxSizeX->setValue(_entryData->_maxSizeX);
-	_maxSizeY->setValue(_entryData->_maxSizeY);
+	_minSize->SetSize(_entryData->_minSize);
+	_maxSize->SetSize(_entryData->_maxSize);
 	_throttleEnable->setChecked(_entryData->_throttleEnabled);
 	_throttleCount->setValue(_entryData->_throttleCount *
 				 GetSwitcher()->interval);
+	_checkAreaEnable->setChecked(_entryData->_checkAreaEnable);
+	_checkArea->SetArea(_entryData->_checkArea);
+	UpdatePreviewTooltip();
 	SetWidgetVisibility();
 }
