@@ -192,6 +192,62 @@ void AdvSceneSwitcher::on_uiHintsDisable_stateChanged(int state)
 	switcher->disableHints = state;
 }
 
+bool isLegacyTab(const QString &name)
+{
+	return name == obs_module_text(
+			       "AdvSceneSwitcher.transitionTab.title") ||
+	       name == obs_module_text(
+			       "AdvSceneSwitcher.windowTitleTab.title") ||
+	       name == obs_module_text(
+			       "AdvSceneSwitcher.executableTab.title") ||
+	       name == obs_module_text(
+			       "AdvSceneSwitcher.screenRegionTab.title") ||
+	       name == obs_module_text("AdvSceneSwitcher.mediaTab.title") ||
+	       name == obs_module_text("AdvSceneSwitcher.fileTab.title") ||
+	       name == obs_module_text("AdvSceneSwitcher.randomTab.title") ||
+	       name == obs_module_text("AdvSceneSwitcher.timeTab.title") ||
+	       name == obs_module_text("AdvSceneSwitcher.idleTab.title") ||
+	       name == obs_module_text(
+			       "AdvSceneSwitcher.sceneSequenceTab.title") ||
+	       name == obs_module_text("AdvSceneSwitcher.audioTab.title") ||
+	       name == obs_module_text("AdvSceneSwitcher.videoTab.title") ||
+	       name == obs_module_text("AdvSceneSwitcher.pauseTab.title") ||
+	       name == obs_module_text(
+			       "AdvSceneSwitcher.sceneTriggerTab.title");
+}
+
+void AdvSceneSwitcher::on_hideLegacyTabs_stateChanged(int state)
+{
+	switcher->hideLegacyTabs = state;
+
+	for (int idx = 0; idx < ui->tabWidget->count(); idx++) {
+		if (isLegacyTab(ui->tabWidget->tabText(idx))) {
+#if defined(_WIN32) || defined(__APPLE__)
+			ui->tabWidget->setTabVisible(idx, !state);
+#else
+			// TODO: Switch to setTabVisible() once QT 5.15 is more wide spread
+			ui->tabWidget->setTabEnabled(idx, !state);
+			ui->tabWidget->setStyleSheet(
+				"QTabBar::tab::disabled {width: 0; height: 0; margin: 0; padding: 0; border: none;} ");
+#endif
+		}
+	}
+}
+
+void AdvSceneSwitcher::setDeprecationWarnings()
+{
+	QString toolTip =
+		switcher->disableHints
+			? ""
+			: obs_module_text(
+				  "AdvSceneSwitcher.deprecatedTabWarning");
+	for (int idx = 0; idx < ui->tabWidget->count(); idx++) {
+		if (isLegacyTab(ui->tabWidget->tabText(idx))) {
+			ui->tabWidget->widget(idx)->setToolTip(toolTip);
+		}
+	}
+}
+
 QString getDefaultSaveLocation()
 {
 	QString desktopPath = QStandardPaths::writableLocation(
@@ -399,6 +455,13 @@ void AdvSceneSwitcher::setTabOrder()
 	connect(bar, &QTabBar::tabMoved, this, &AdvSceneSwitcher::on_tabMoved);
 }
 
+void AdvSceneSwitcher::setCurrentTab()
+{
+	if (switcher->lastOpenedTab >= 0) {
+		ui->tabWidget->setCurrentIndex(switcher->lastOpenedTab);
+	}
+}
+
 void AdvSceneSwitcher::restoreWindowGeo()
 {
 	if (switcher->saveWindowGeo && windowPosValid(switcher->windowPos)) {
@@ -410,6 +473,7 @@ void AdvSceneSwitcher::restoreWindowGeo()
 void AdvSceneSwitcher::checkFirstTimeSetup()
 {
 	if (switcher->firstBoot && !switcher->disableHints) {
+		switcher->firstBoot = false;
 		DisplayMessage(
 			obs_module_text("AdvSceneSwitcher.firstBootMessage"));
 	}
@@ -456,6 +520,9 @@ void SwitcherData::loadSettings(obs_data_t *obj)
 	loadSceneTriggers(obj);
 	loadGeneralSettings(obj);
 	loadHotkeys(obj);
+
+	// Reset on startup and scene collection change
+	switcher->lastOpenedTab = -1;
 }
 
 void SwitcherData::saveSettings(obs_data_t *obj)
@@ -511,6 +578,7 @@ void SwitcherData::saveGeneralSettings(obs_data_t *obj)
 	obs_data_set_bool(obj, "showSystemTrayNotifications",
 			  showSystemTrayNotifications);
 	obs_data_set_bool(obj, "disableHints", disableHints);
+	obs_data_set_bool(obj, "hideLegacyTabs", hideLegacyTabs);
 
 	obs_data_set_int(obj, "priority0", functionNamesByPriority[0]);
 	obs_data_set_int(obj, "priority1", functionNamesByPriority[1]);
@@ -584,6 +652,8 @@ void SwitcherData::loadGeneralSettings(obs_data_t *obj)
 	showSystemTrayNotifications =
 		obs_data_get_bool(obj, "showSystemTrayNotifications");
 	disableHints = obs_data_get_bool(obj, "disableHints");
+	obs_data_set_default_bool(obj, "hideLegacyTabs", true);
+	hideLegacyTabs = obs_data_get_bool(obj, "hideLegacyTabs");
 
 	obs_data_set_default_int(obj, "priority0", default_priority_0);
 	obs_data_set_default_int(obj, "priority1", default_priority_1);
@@ -626,21 +696,21 @@ void SwitcherData::loadGeneralSettings(obs_data_t *obj)
 
 	obs_data_set_default_int(obj, "generalTabPos", 0);
 	obs_data_set_default_int(obj, "macroTabPos", 1);
-	obs_data_set_default_int(obj, "transitionTabPos", 2);
-	obs_data_set_default_int(obj, "pauseTabPos", 3);
-	obs_data_set_default_int(obj, "titleTabPos", 4);
-	obs_data_set_default_int(obj, "exeTabPos", 5);
-	obs_data_set_default_int(obj, "regionTabPos", 6);
-	obs_data_set_default_int(obj, "mediaTabPos", 7);
-	obs_data_set_default_int(obj, "fileTabPos", 8);
-	obs_data_set_default_int(obj, "randomTabPos", 9);
-	obs_data_set_default_int(obj, "timeTabPos", 10);
-	obs_data_set_default_int(obj, "idleTabPos", 11);
-	obs_data_set_default_int(obj, "sequenceTabPos", 12);
-	obs_data_set_default_int(obj, "audioTabPos", 13);
-	obs_data_set_default_int(obj, "videoTabPos", 14);
-	obs_data_set_default_int(obj, "networkTabPos", 15);
-	obs_data_set_default_int(obj, "sceneGroupTabPos", 16);
+	obs_data_set_default_int(obj, "networkTabPos", 13);
+	obs_data_set_default_int(obj, "sceneGroupTabPos", 14);
+	obs_data_set_default_int(obj, "transitionTabPos", 15);
+	obs_data_set_default_int(obj, "pauseTabPos", 16);
+	obs_data_set_default_int(obj, "titleTabPos", 2);
+	obs_data_set_default_int(obj, "exeTabPos", 3);
+	obs_data_set_default_int(obj, "regionTabPos", 4);
+	obs_data_set_default_int(obj, "mediaTabPos", 5);
+	obs_data_set_default_int(obj, "fileTabPos", 6);
+	obs_data_set_default_int(obj, "randomTabPos", 7);
+	obs_data_set_default_int(obj, "timeTabPos", 8);
+	obs_data_set_default_int(obj, "idleTabPos", 9);
+	obs_data_set_default_int(obj, "sequenceTabPos", 10);
+	obs_data_set_default_int(obj, "audioTabPos", 11);
+	obs_data_set_default_int(obj, "videoTabPos", 12);
 	obs_data_set_default_int(obj, "triggerTabPos", 17);
 
 	tabOrder.clear();
@@ -797,6 +867,7 @@ void AdvSceneSwitcher::setupGeneralTab()
 	ui->showTrayNotifications->setChecked(
 		switcher->showSystemTrayNotifications);
 	ui->uiHintsDisable->setChecked(switcher->disableHints);
+	ui->hideLegacyTabs->setChecked(switcher->hideLegacyTabs);
 
 	for (int p : switcher->functionNamesByPriority) {
 		std::string s = "";
