@@ -1,5 +1,8 @@
 #pragma once
-#include "macro.hpp"
+#include "macro-action.hpp"
+
+#include <QComboBox>
+#include <memory>
 
 struct MacroActionInfo {
 	using TCreateMethod = std::shared_ptr<MacroAction> (*)(Macro *m);
@@ -34,6 +37,7 @@ public:
 			std::shared_ptr<MacroAction> * = nullptr,
 			const std::string &id = "scene_switch");
 	void UpdateEntryData(const std::string &id);
+	void SetEntryData(std::shared_ptr<MacroAction> *);
 
 private slots:
 	void ActionSelectionChanged(const QString &text);
