@@ -4,10 +4,13 @@
 #include <QImage>
 #include <chrono>
 
-class AdvSSScreenshotObj {
+class ScreenshotHelper {
 public:
-	AdvSSScreenshotObj(obs_source_t *source);
-	~AdvSSScreenshotObj();
+	ScreenshotHelper() = default;
+	ScreenshotHelper(obs_source_t *source);
+	ScreenshotHelper &operator=(const ScreenshotHelper &) = delete;
+	ScreenshotHelper(const ScreenshotHelper &) = delete;
+	~ScreenshotHelper();
 
 	void Screenshot();
 	void Download();
@@ -17,7 +20,6 @@ public:
 	gs_texrender_t *texrender = nullptr;
 	gs_stagesurf_t *stagesurf = nullptr;
 	OBSWeakSource weakSource;
-	std::string path;
 	QImage image;
 	uint32_t cx = 0;
 	uint32_t cy = 0;
@@ -26,4 +28,7 @@ public:
 
 	bool done = false;
 	std::chrono::high_resolution_clock::time_point time;
+
+private:
+	bool _initDone = false;
 };
