@@ -204,12 +204,13 @@ void placeWidgets(std::string text, QBoxLayout *layout,
 	}
 }
 
-void deleteLayoutItem(QLayoutItem *item)
+void deleteLayoutItemWidget(QLayoutItem *item)
 {
 	if (item) {
 		auto widget = item->widget();
 		if (widget) {
 			widget->setVisible(false);
+			widget->deleteLater();
 		}
 		delete item;
 	}
@@ -223,20 +224,24 @@ void clearLayout(QLayout *layout, int afterIdx)
 			clearLayout(item->layout());
 			delete item->layout();
 		}
-		if (item->widget()) {
-			delete item->widget();
-		}
-		delete item;
+		deleteLayoutItemWidget(item);
 	}
 }
 
 void setLayoutVisible(QLayout *layout, bool visible)
 {
+	if (!layout) {
+		return;
+	}
 	for (int i = 0; i < layout->count(); ++i) {
 		QWidget *widget = layout->itemAt(i)->widget();
-		if (widget != NULL) {
+		QLayout *nestedLayout = layout->itemAt(i)->layout();
+		if (widget) {
 			widget->setVisible(visible);
 		}
+		if (nestedLayout) {
+			setLayoutVisible(nestedLayout, visible);
+		}
 	}
 }
 
@@ -595,7 +600,8 @@ void populateAudioSelection(QComboBox *sel, bool addSelect)
 	sel->setCurrentIndex(0);
 }
 
-void populateVideoSelection(QComboBox *sel, bool addScenes, bool addSelect)
+void populateVideoSelection(QComboBox *sel, bool addMainOutput, bool addScenes,
+			    bool addSelect)
 {
 
 	auto sourceEnum = [](void *data, obs_source_t *source) -> bool /* -- */
@@ -623,6 +629,10 @@ void populateVideoSelection(QComboBox *sel, bool addScenes, bool addSelect)
 	}
 
 	sel->model()->sort(0);
+	if (addMainOutput) {
+		sel->insertItem(
+			0, obs_module_text("AdvSceneSwitcher.OBSVideoOutput"));
+	}
 	if (addSelect) {
 		addSelectionEntry(
 			sel,
@@ -881,37 +891,42 @@ bool windowPosValid(QPoint pos)
 }
 
 QMetaObject::Connection PulseWidget(QWidget *widget, QColor startColor,
-				    QColor endColor, QString specifier,
-				    bool once)
+				    QColor endColor, bool once)
 {
-	widget->setStyleSheet(specifier + "{ \
-		border-style: outset; \
-		border-width: 2px; \
-		border-radius: 10px; \
-		border-color: rgb(0,0,0,0) \
-		}");
-
-	QGraphicsColorizeEffect *eEffect = new QGraphicsColorizeEffect(widget);
-	widget->setGraphicsEffect(eEffect);
-	QPropertyAnimation *paAnimation =
-		new QPropertyAnimation(eEffect, "color");
-	paAnimation->setStartValue(startColor);
-	paAnimation->setEndValue(endColor);
-	paAnimation->setDuration(1000);
+	QGraphicsColorizeEffect *effect = new QGraphicsColorizeEffect(widget);
+	widget->setGraphicsEffect(effect);
+	QPropertyAnimation *animation =
+		new QPropertyAnimation(effect, "color", widget);
+	animation->setStartValue(startColor);
+	animation->setEndValue(endColor);
+	animation->setDuration(1000);
 
 	QMetaObject::Connection con;
-	if (!once) {
+	if (once) {
+		auto widgetPtr = widget;
 		con = QWidget::connect(
-			paAnimation, &QPropertyAnimation::finished,
-			[paAnimation]() {
-				QTimer::singleShot(1000, [paAnimation] {
-					paAnimation->start();
-				});
+			animation, &QPropertyAnimation::finished,
+			[widgetPtr]() {
+				if (widgetPtr) {
+					widgetPtr->setGraphicsEffect(nullptr);
+				}
 			});
+		animation->start(QPropertyAnimation::DeleteWhenStopped);
+	} else {
+		auto widgetPtr = widget;
+		con = QWidget::connect(
+			animation, &QPropertyAnimation::finished,
+			[animation, widgetPtr]() {
+				QTimer *timer = new QTimer(widgetPtr);
+				QWidget::connect(timer, &QTimer::timeout,
+						 [animation] {
+							 animation->start();
+						 });
+				timer->setSingleShot(true);
+				timer->start(1000);
+			});
+		animation->start();
 	}
-
-	paAnimation->start();
-
 	return con;
 }
 
@@ -984,3 +999,8 @@ void setHeightToContentHeight(QListWidget *list)
 				       2 * list->frameWidth());
 	}
 }
+
+bool doubleEquals(double left, double right, double epsilon)
+{
+	return (fabs(left - right) < epsilon);
+}
